// file bin_am_prop_div_call.cc
// author: Bernt Arne Oedegaard
// binomial option pricing adjusting for dividends.

#include <cmath>
#include <algorithm>
#include <vector>
#include "fin_algoritms.h"

double option_price_call_american_proportional_dividends_binomial(
   double S, 
   double X,
   double r,
   double sigma, 
   double time, 
   int no_steps,
   vector<double>& dividend_times,
   vector<double>& dividend_yields) 
// given a dividend yield, the binomial tree recombines 
{
   int no_dividends=dividend_times.size();
   if (no_dividends == 0)               // just take the regular binomial 
      return option_price_call_american_binomial(S,X,r,sigma,time,no_steps);
   double R = exp(r*(time/no_steps));
   double Rinv = 1.0/R;
   double u = exp(sigma*sqrt(time/no_steps));
   double uu= u*u;
   double d = 1.0/u;
   double pUp   = (R-d)/(u-d);
   double pDown = 1.0 - pUp;
   
   vector<int> dividend_steps(no_dividends); // when dividends are paid
   for (int i=0; i<no_dividends; ++i) {
      dividend_steps[i] = (int)(dividend_times[i]/time*no_steps);
   };
   vector<double> prices(no_steps+1);
   vector<double> call_prices(no_steps+1);
   prices[0] = S*pow(d, no_steps);
   for (int i=0; i<no_dividends; ++i) { prices[0]*=(1.0-dividend_yields[i]); };
   for (int i=1; i<=no_steps; ++i) prices[i] = uu*prices[i-1]; // terminal tree nodes
   for (int i=0; i<=no_steps; ++i) call_prices[i] = max(0.0, (prices[i]-X));
   for (int step=no_steps-1; step>=0; --step) {
      for (int i=0;i<no_dividends;++i) {   // check whether dividend paid
	 if (step==dividend_steps[i]) { 
	    for (int j=0;j<=step;++j) {
	       prices[j]*=(1.0/(1.0-dividend_yields[i])); 
	    };
	 };
      };
      for (int i=0; i<=step; ++i)   {
	 prices[i] = d*prices[i+1];
	 call_prices[i] = (pDown*call_prices[i]+pUp*call_prices[i+1])*Rinv;
	 call_prices[i] = max(call_prices[i], prices[i]-X);         // check for exercise
      };
   };
   return call_prices[0];
};

