// file mv_calc_portfolio_constrained.cc 
// author: Bernt Arne Oedegaard
// calculate a mean variance portfolio under short sales constraints

#include "mv_calc.h"
#include "cfsqp_op.h"   // optimize using CFSQP
#include "mymatrix.h"

int      MV_CONSTRAINED_no_assets;   // global values because they need to be 
double   MV_CONSTRAINED_r;           // available in criterion function
Matrix*  MV_CONSTRAINED_e;
Matrix*  MV_CONSTRAINED_V;

double MV_CONSTRAINED_objective(double x[]){
    Matrix X(MV_CONSTRAINED_no_assets,1);
    for (int i=0;i<MV_CONSTRAINED_no_assets;++i){
	X.element(i,0)=x[i];
    };
    return mv_calculate_variance(*MV_CONSTRAINED_V,X);
}

double MV_CONSTRAINED_constraints(double x[], int constraint_no){
    Matrix X(MV_CONSTRAINED_no_assets,1);
    for (int i=0;i<MV_CONSTRAINED_no_assets;++i){
	X.element(i,0)=x[i];
    };
    if (constraint_no==1) {    // constraint w'1=1
	Matrix A = (X.t()*ones_matrix(MV_CONSTRAINED_no_assets,1));
	return A.element(0,0)-1.0;
    };
    if (constraint_no==2) {    // constraint w'e=r
	Matrix A = (X.t()*(*MV_CONSTRAINED_e));
	return A.element(0,0)-MV_CONSTRAINED_r;
    };
    return 0;
};

ReturnMatrix mv_calculate_portfolio_given_mean_min_max_constraints(
    Matrix& e,
    Matrix& V,
    const vector<double>& min_weights,
    const vector<double>& max_weights,
    double r)
    // solve the quadratic program to minimize variance given constraints
    // all weights are in [0,1] 
    // the weights sum to one
    // expected return equals r
{
    MV_CONSTRAINED_no_assets = e.Nrows();
    MV_CONSTRAINED_V = &V;
    MV_CONSTRAINED_e = &e;
    MV_CONSTRAINED_r = r;
    Matrix w=null_matrix(MV_CONSTRAINED_no_assets, 1);
    // check whether it is feasible to find optimum. Only if r is between 
    // min and max in e;
    double r_min = e.element(0,0);
    double r_max = e.element(0,0);
    int i_min = 0;
    int i_max = 0;
    int i;
    for (i=1;i<MV_CONSTRAINED_no_assets;++i) {
	if (e.element(i,0)<r_min) { 
	    r_min = e.element(i,0); 
	    i_min = i;
	};
	if (e.element(i,0)>r_max) { 
	    r_max = e.element(i,0); 
	    i_max = i;
	};
    };
    if ( (r>r_max) || (r<r_min) ) {
	cerr << " can not perform optimization " << endl;
	w.Release(); // returning zeros as weight
	return w;
    };     
    double *x = new double[MV_CONSTRAINED_no_assets];
    double *x_min = new double[MV_CONSTRAINED_no_assets];
    double *x_max = new double[MV_CONSTRAINED_no_assets];
    for (i=0;i<MV_CONSTRAINED_no_assets;++i) {
	x[i]      = 0;
	x_min[i]  = min_weights[i];
	x_max[i]  = max_weights[i];
    };
    // now generate feasible portfolio by combining r_max and r_min;
    double lambda = (r-r_max)/(r_min-r_max);
    x[i_min]=lambda;
    x[i_max]=(1-lambda);
    double min_var =0;
    min_var = CFSQP_optimize_with_constraints_and_bounds ( 
       x,
       MV_CONSTRAINED_no_assets, 
       MV_CONSTRAINED_objective,
       0,0,0,2,                         // 2 linear equality constraints
       MV_CONSTRAINED_constraints, 
       x_min,
       x_max);
    for (i=0; i<MV_CONSTRAINED_no_assets; ++i) {
       w.element(i,0)=x[i]; 
    };
    delete [] x;
    delete [] x_min;
    delete [] x_max;
    w.Release();
    return w;
};
